"""
Copyright (c) 2014 Maciej Nabozny

This file is part of CloudOver project.

CloudOver is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from django.http import HttpResponse

import netaddr

from corecluster.models.core.lease import Lease
from corecluster.models.core.network_pool import NetworkPool
from coretalk.models.coretalk.userdata import UserData
from coretalk.models.coretalk.sshkey import SshKey

def resolve_ip(remote_ip):
    addr = netaddr.IPAddress(remote_ip)
    for network in NetworkPool.objects.all():
        if addr >= network.to_ipnetwork().network and addr <= network.to_ipnetwork().broadcast:
            try:
                if network.mode == 'routed':
                    return Lease.objects.get(address=str(addr-2))
                elif network.mode == 'isolated':
                    return Lease.objects.get(address=remote_ip)
            except:
                continue

    raise Exception('lease_not_found')


def get_root(request):
    return HttpResponse('latest\n')

def get_latest(request):
    return HttpResponse('meta-data\nuser-data\n')

def get_meta_data(request):
    return HttpResponse('ami-id\nhostname\ninstance-id\ninstance-type\npublic-keys\n')


# Meta-data Functions
def get_ami_id(request):
    try:
        lease = resolve_ip(request.META.get('REMOTE_ADDR'))
    except:
        return HttpResponse('none')

    vm = lease.vm
    if vm == None:
        return HttpResponse('none')

    if vm.base_image == None:
        return HttpResponse('none')

    return HttpResponse(vm.base_image.id)


def get_hostname(request):
    try:
        lease = resolve_ip(request.META.get('REMOTE_ADDR'))
    except:
        return HttpResponse('none')

    return HttpResponse(lease.hostname)


def get_instance_id(request):
    try:
        lease = resolve_ip(request.META.get('REMOTE_ADDR'))
    except:
        return HttpResponse('none')

    vm = lease.vm
    if vm == None:
        return HttpResponse('none')

    return HttpResponse(vm.id)


def get_instance_type(request):
    try:
        lease = resolve_ip(request.META.get('REMOTE_ADDR'))
    except:
        return HttpResponse('none')

    vm = lease.vm
    if vm == None:
        return HttpResponse('none')

    return HttpResponse(vm.template.ec2name)


def get_public_keys(request):
    try:
        lease = resolve_ip(request.META.get('REMOTE_ADDR'))
    except:
        return HttpResponse('none')

    vm = lease.vm
    if vm == None:
        return HttpResponse('none')

    try:
        keys = vm.get_prop("coretalk_keys")
        key_list = ''
        for k in keys:
            key = SshKey.objects.get(id=k)
            key_list += key.id + '=' + key.name + '\n'
        return HttpResponse(key_list)
    except:
        return HttpResponse('none')


def get_ssh_key_formats(request, id):
    try:
        lease = resolve_ip(request.META.get('REMOTE_ADDR'))
    except:
        return HttpResponse('none')

    vm = lease.vm
    if vm == None:
        return HttpResponse('none')

    print "openssh-key"


def get_ssh_key_formats(request, id):
    try:
        lease = resolve_ip(request.META.get('REMOTE_ADDR'))
    except:
        return HttpResponse('none')

    vm = lease.vm
    if vm == None:
        return HttpResponse('none')

    key = SshKey.objects.get(id=id)
    return HttpResponse(key.key)


def get_user_data(request):
    try:
        lease = resolve_ip(request.META.get('REMOTE_ADDR'))
    except:
        return HttpResponse('none')

    vm = lease.vm
    if vm == None:
        return HttpResponse('none')

    try:
        user_data_id = vm.get_prop('coretalk_user_data')
    except:
        return HttpResponse('none')

    try:
        user_data = UserData.objects.get(id=user_data_id)
    except:
        user_data = ''

    return HttpResponse(user_data.data)